# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'gravtools/gui/dialog_load_cg6_obs_files.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DialogLoadCg6ObservationFiles(object):
    def setupUi(self, DialogLoadCg6ObservationFiles):
        DialogLoadCg6ObservationFiles.setObjectName("DialogLoadCg6ObservationFiles")
        DialogLoadCg6ObservationFiles.resize(679, 529)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(DialogLoadCg6ObservationFiles)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.groupBox_format_options = QtWidgets.QGroupBox(DialogLoadCg6ObservationFiles)
        self.groupBox_format_options.setObjectName("groupBox_format_options")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox_format_options)
        self.verticalLayout.setObjectName("verticalLayout")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.comboBox_format_selection = QtWidgets.QComboBox(self.groupBox_format_options)
        self.comboBox_format_selection.setObjectName("comboBox_format_selection")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.comboBox_format_selection)
        self.label_format_selection = QtWidgets.QLabel(self.groupBox_format_options)
        self.label_format_selection.setObjectName("label_format_selection")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.label_format_selection)
        self.spinBox_dt_sec = QtWidgets.QSpinBox(self.groupBox_format_options)
        self.spinBox_dt_sec.setEnabled(True)
        self.spinBox_dt_sec.setMaximum(9999)
        self.spinBox_dt_sec.setProperty("value", 600)
        self.spinBox_dt_sec.setObjectName("spinBox_dt_sec")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.spinBox_dt_sec)
        self.label_dt_sec = QtWidgets.QLabel(self.groupBox_format_options)
        self.label_dt_sec.setEnabled(True)
        self.label_dt_sec.setObjectName("label_dt_sec")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.label_dt_sec)
        self.checkBox_use_dt = QtWidgets.QCheckBox(self.groupBox_format_options)
        self.checkBox_use_dt.setText("")
        self.checkBox_use_dt.setObjectName("checkBox_use_dt")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.checkBox_use_dt)
        self.label_use_dt = QtWidgets.QLabel(self.groupBox_format_options)
        self.label_use_dt.setObjectName("label_use_dt")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.label_use_dt)
        self.label = QtWidgets.QLabel(self.groupBox_format_options)
        self.label.setToolTip("")
        self.label.setObjectName("label")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label)
        self.label_format_description = QtWidgets.QLabel(self.groupBox_format_options)
        self.label_format_description.setText("")
        self.label_format_description.setWordWrap(True)
        self.label_format_description.setObjectName("label_format_description")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.label_format_description)
        self.verticalLayout.addLayout(self.formLayout)
        self.verticalLayout_2.addWidget(self.groupBox_format_options)
        self.groupBox_content_options = QtWidgets.QGroupBox(DialogLoadCg6ObservationFiles)
        self.groupBox_content_options.setObjectName("groupBox_content_options")
        self.formLayout_2 = QtWidgets.QFormLayout(self.groupBox_content_options)
        self.formLayout_2.setObjectName("formLayout_2")
        self.label_location_type = QtWidgets.QLabel(self.groupBox_content_options)
        self.label_location_type.setObjectName("label_location_type")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_location_type)
        self.comboBox_location_type = QtWidgets.QComboBox(self.groupBox_content_options)
        self.comboBox_location_type.setObjectName("comboBox_location_type")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.comboBox_location_type)
        self.label_obs_uncertainty_type = QtWidgets.QLabel(self.groupBox_content_options)
        self.label_obs_uncertainty_type.setObjectName("label_obs_uncertainty_type")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_obs_uncertainty_type)
        self.comboBox_uncertainty_type = QtWidgets.QComboBox(self.groupBox_content_options)
        self.comboBox_uncertainty_type.setObjectName("comboBox_uncertainty_type")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.comboBox_uncertainty_type)
        self.label_pres_in_column = QtWidgets.QLabel(self.groupBox_content_options)
        self.label_pres_in_column.setObjectName("label_pres_in_column")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_pres_in_column)
        self.comboBox_pres_in_column = QtWidgets.QComboBox(self.groupBox_content_options)
        self.comboBox_pres_in_column.setObjectName("comboBox_pres_in_column")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.comboBox_pres_in_column)
        self.label_dhb_in_column = QtWidgets.QLabel(self.groupBox_content_options)
        self.label_dhb_in_column.setObjectName("label_dhb_in_column")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.label_dhb_in_column)
        self.comboBox_dhb_in_column = QtWidgets.QComboBox(self.groupBox_content_options)
        self.comboBox_dhb_in_column.setObjectName("comboBox_dhb_in_column")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.comboBox_dhb_in_column)
        self.verticalLayout_2.addWidget(self.groupBox_content_options)
        self.listWidget_open_files = QtWidgets.QListWidget(DialogLoadCg6ObservationFiles)
        self.listWidget_open_files.setObjectName("listWidget_open_files")
        self.verticalLayout_2.addWidget(self.listWidget_open_files)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.pushButton_select_files = QtWidgets.QPushButton(DialogLoadCg6ObservationFiles)
        self.pushButton_select_files.setObjectName("pushButton_select_files")
        self.horizontalLayout_2.addWidget(self.pushButton_select_files)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.pushButton_clear_list = QtWidgets.QPushButton(DialogLoadCg6ObservationFiles)
        self.pushButton_clear_list.setObjectName("pushButton_clear_list")
        self.horizontalLayout_2.addWidget(self.pushButton_clear_list)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.buttonBox = QtWidgets.QDialogButtonBox(DialogLoadCg6ObservationFiles)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout_2.addWidget(self.buttonBox)

        self.retranslateUi(DialogLoadCg6ObservationFiles)
        self.buttonBox.accepted.connect(DialogLoadCg6ObservationFiles.accept) # type: ignore
        self.buttonBox.rejected.connect(DialogLoadCg6ObservationFiles.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(DialogLoadCg6ObservationFiles)

    def retranslateUi(self, DialogLoadCg6ObservationFiles):
        _translate = QtCore.QCoreApplication.translate
        DialogLoadCg6ObservationFiles.setWindowTitle(_translate("DialogLoadCg6ObservationFiles", "Load CG-6 observation files"))
        self.groupBox_format_options.setToolTip(_translate("DialogLoadCg6ObservationFiles", "File format options."))
        self.groupBox_format_options.setTitle(_translate("DialogLoadCg6ObservationFiles", "Format options"))
        self.comboBox_format_selection.setToolTip(_translate("DialogLoadCg6ObservationFiles", "Select the CG6 observation file format."))
        self.label_format_selection.setToolTip(_translate("DialogLoadCg6ObservationFiles", "Select the CG6 observation file format."))
        self.label_format_selection.setText(_translate("DialogLoadCg6ObservationFiles", "CG-6 observation file format"))
        self.spinBox_dt_sec.setToolTip(_translate("DialogLoadCg6ObservationFiles", "Time gap that is used to divide observations into differnt setups."))
        self.label_dt_sec.setToolTip(_translate("DialogLoadCg6ObservationFiles", "Time gap that is used to divide observations into differnt setups."))
        self.label_dt_sec.setText(_translate("DialogLoadCg6ObservationFiles", "Minimum time gap to between setups [sec]"))
        self.checkBox_use_dt.setToolTip(_translate("DialogLoadCg6ObservationFiles", "If the time gap between consecutive observations is larger than the value defined below, these observationsare assignet to different instrumet setups."))
        self.label_use_dt.setToolTip(_translate("DialogLoadCg6ObservationFiles", "If the time gap between consecutive observations is larger than the value defined below, these observationsare assignet to different instrumet setups."))
        self.label_use_dt.setText(_translate("DialogLoadCg6ObservationFiles", "Divide observations into different setups based on the specified time gap"))
        self.label.setText(_translate("DialogLoadCg6ObservationFiles", "Format description"))
        self.groupBox_content_options.setToolTip(_translate("DialogLoadCg6ObservationFiles", "File content options. "))
        self.groupBox_content_options.setTitle(_translate("DialogLoadCg6ObservationFiles", "Content options"))
        self.label_location_type.setToolTip(_translate("DialogLoadCg6ObservationFiles", "<html><head/><body><p>Select the type of location data (i.e. the location source) that is used for georeferencing the observations. The CG6 observation file format contains location data based on user input (&quot;<span style=\" font-weight:600;\">user</span>&quot;) and based on GPS measurements (&quot;<span style=\" font-weight:600;\">GPS</span>&quot;). Please note, that GPS locations might not be available in the observation files, e.g. at stations without GPS signal availability.</p></body></html>"))
        self.label_location_type.setText(_translate("DialogLoadCg6ObservationFiles", "Location data source"))
        self.comboBox_location_type.setToolTip(_translate("DialogLoadCg6ObservationFiles", "<html><head/><body><p>Select the type of location data (i.e. the location source) that is used for georeferencing the observations. The CG6 observation file format contains location data based on user input (&quot;<span style=\" font-weight:600;\">user</span>&quot;) and based on GPS measurements (&quot;<span style=\" font-weight:600;\">GPS</span>&quot;). Please note, that GPS locations might not be available in the observation files, e.g. at stations without GPS signal availability.</p></body></html>"))
        self.label_obs_uncertainty_type.setToolTip(_translate("DialogLoadCg6ObservationFiles", "<html><head/><body><p>Select the type of observation uncertainty that is loaded from the observation files. The CG6 provides two types of uncertainties: <span style=\" font-weight:600;\">standard deviations</span> and <span style=\" font-weight:600;\">standard errors</span>. Have a look at the instrument\'s manual for more details.</p></body></html>"))
        self.label_obs_uncertainty_type.setText(_translate("DialogLoadCg6ObservationFiles", "Obs. uncertainty type"))
        self.comboBox_uncertainty_type.setToolTip(_translate("DialogLoadCg6ObservationFiles", "<html><head/><body><p>Select the type of observation uncertainty that is loaded from the observation files. The CG6 provides two types of uncertainties: <span style=\" font-weight:600;\">standard deviations</span> and <span style=\" font-weight:600;\">standard errors</span>. Have a look at the instrument\'s manual for more details.</p></body></html>"))
        self.label_pres_in_column.setToolTip(_translate("DialogLoadCg6ObservationFiles", "<html><head/><body><p>Select the observation file colum containing in-situ measurements of the atmospheric pressure [hPa]. </p><p>Since the CG6 observation file formats do not provide a native data column for this kind of information, another column with user input can be used instead.</p></body></html>"))
        self.label_pres_in_column.setText(_translate("DialogLoadCg6ObservationFiles", "Pressure [hPa] in column"))
        self.comboBox_pres_in_column.setToolTip(_translate("DialogLoadCg6ObservationFiles", "<html><head/><body><p>Select the observation file colum containing in-situ measurements of the atmospheric pressure [hPa]. </p><p>Since the CG6 observation file formats do not provide a native data column for this kind of information, another column with user input can be used instead.</p></body></html>"))
        self.label_dhb_in_column.setToolTip(_translate("DialogLoadCg6ObservationFiles", "<html><head/><body><p>Select the observation file colum containing height differences [m] between the instrument\'s top surface and the ground.</p><p>Since the CG6 observation file formats do not provide a native data column for this kind of information, another column with user input can be used instead.</p></body></html>"))
        self.label_dhb_in_column.setText(_translate("DialogLoadCg6ObservationFiles", "dh to ground [m] in column"))
        self.comboBox_dhb_in_column.setToolTip(_translate("DialogLoadCg6ObservationFiles", "<html><head/><body><p>Select the observation file colum containing height differences [m] between the instrument\'s top surface and the ground.</p><p>Since the CG6 observation file formats do not provide a native data column for this kind of information, another column with user input can be used instead.</p></body></html>"))
        self.listWidget_open_files.setToolTip(_translate("DialogLoadCg6ObservationFiles", "<html><head/><body><p>List of observation files that will be loaded. Please be aware that all files need to have the same format (matching the selection above.</p></body></html>"))
        self.pushButton_select_files.setToolTip(_translate("DialogLoadCg6ObservationFiles", "<html><head/><body><p>Press to open file selection dialog for observation files. Please be aware that all selectd files need to have have the same format (matching the selection above).</p></body></html>"))
        self.pushButton_select_files.setText(_translate("DialogLoadCg6ObservationFiles", "Select observation files"))
        self.pushButton_clear_list.setToolTip(_translate("DialogLoadCg6ObservationFiles", "Press to clear the observation file list."))
        self.pushButton_clear_list.setText(_translate("DialogLoadCg6ObservationFiles", "Clear list"))
        self.buttonBox.setToolTip(_translate("DialogLoadCg6ObservationFiles", "Press OK to load the observation files in the list above."))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'gravtools/gui/dialog_corrections.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog_corrections(object):
    def setupUi(self, Dialog_corrections):
        Dialog_corrections.setObjectName("Dialog_corrections")
        Dialog_corrections.resize(481, 639)
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(Dialog_corrections)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.tabWidget = QtWidgets.QTabWidget(Dialog_corrections)
        self.tabWidget.setObjectName("tabWidget")
        self.tab_all_instruments = QtWidgets.QWidget()
        self.tab_all_instruments.setObjectName("tab_all_instruments")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.tab_all_instruments)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.groupBox_corrections_tides = QtWidgets.QGroupBox(self.tab_all_instruments)
        self.groupBox_corrections_tides.setObjectName("groupBox_corrections_tides")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_corrections_tides)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.radioButton_corr_tides_no_correction = QtWidgets.QRadioButton(self.groupBox_corrections_tides)
        self.radioButton_corr_tides_no_correction.setObjectName("radioButton_corr_tides_no_correction")
        self.buttonGroup_corrections_tides = QtWidgets.QButtonGroup(Dialog_corrections)
        self.buttonGroup_corrections_tides.setObjectName("buttonGroup_corrections_tides")
        self.buttonGroup_corrections_tides.addButton(self.radioButton_corr_tides_no_correction)
        self.verticalLayout.addWidget(self.radioButton_corr_tides_no_correction)
        self.radioButton_corr_tides_cg5_model = QtWidgets.QRadioButton(self.groupBox_corrections_tides)
        self.radioButton_corr_tides_cg5_model.setChecked(True)
        self.radioButton_corr_tides_cg5_model.setObjectName("radioButton_corr_tides_cg5_model")
        self.buttonGroup_corrections_tides.addButton(self.radioButton_corr_tides_cg5_model)
        self.verticalLayout.addWidget(self.radioButton_corr_tides_cg5_model)
        self.radioButton_corr_tides_longman1959 = QtWidgets.QRadioButton(self.groupBox_corrections_tides)
        self.radioButton_corr_tides_longman1959.setObjectName("radioButton_corr_tides_longman1959")
        self.buttonGroup_corrections_tides.addButton(self.radioButton_corr_tides_longman1959)
        self.verticalLayout.addWidget(self.radioButton_corr_tides_longman1959)
        self.radioButton_corr_tides_time_series = QtWidgets.QRadioButton(self.groupBox_corrections_tides)
        self.radioButton_corr_tides_time_series.setObjectName("radioButton_corr_tides_time_series")
        self.buttonGroup_corrections_tides.addButton(self.radioButton_corr_tides_time_series)
        self.verticalLayout.addWidget(self.radioButton_corr_tides_time_series)
        self.verticalLayout_3.addLayout(self.verticalLayout)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.label_tides_interpolation_method = QtWidgets.QLabel(self.groupBox_corrections_tides)
        self.label_tides_interpolation_method.setEnabled(False)
        self.label_tides_interpolation_method.setObjectName("label_tides_interpolation_method")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_tides_interpolation_method)
        self.comboBox_tides_interpolation_method = QtWidgets.QComboBox(self.groupBox_corrections_tides)
        self.comboBox_tides_interpolation_method.setEnabled(False)
        self.comboBox_tides_interpolation_method.setObjectName("comboBox_tides_interpolation_method")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.comboBox_tides_interpolation_method)
        self.verticalLayout_3.addLayout(self.formLayout)
        self.verticalLayout_7.addWidget(self.groupBox_corrections_tides)
        self.groupBox_corrections_ref_heights = QtWidgets.QGroupBox(self.tab_all_instruments)
        self.groupBox_corrections_ref_heights.setObjectName("groupBox_corrections_ref_heights")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.groupBox_corrections_ref_heights)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.radioButton_corr_ref_heights_sensor = QtWidgets.QRadioButton(self.groupBox_corrections_ref_heights)
        self.radioButton_corr_ref_heights_sensor.setObjectName("radioButton_corr_ref_heights_sensor")
        self.buttonGroup_corrections_ref_heights = QtWidgets.QButtonGroup(Dialog_corrections)
        self.buttonGroup_corrections_ref_heights.setObjectName("buttonGroup_corrections_ref_heights")
        self.buttonGroup_corrections_ref_heights.addButton(self.radioButton_corr_ref_heights_sensor)
        self.verticalLayout_2.addWidget(self.radioButton_corr_ref_heights_sensor)
        self.radioButton_corr_ref_heights_instrument_top = QtWidgets.QRadioButton(self.groupBox_corrections_ref_heights)
        self.radioButton_corr_ref_heights_instrument_top.setObjectName("radioButton_corr_ref_heights_instrument_top")
        self.buttonGroup_corrections_ref_heights.addButton(self.radioButton_corr_ref_heights_instrument_top)
        self.verticalLayout_2.addWidget(self.radioButton_corr_ref_heights_instrument_top)
        self.radioButton_corr_ref_heights_ground = QtWidgets.QRadioButton(self.groupBox_corrections_ref_heights)
        self.radioButton_corr_ref_heights_ground.setObjectName("radioButton_corr_ref_heights_ground")
        self.buttonGroup_corrections_ref_heights.addButton(self.radioButton_corr_ref_heights_ground)
        self.verticalLayout_2.addWidget(self.radioButton_corr_ref_heights_ground)
        self.radioButton_corr_ref_heights_control_point = QtWidgets.QRadioButton(self.groupBox_corrections_ref_heights)
        self.radioButton_corr_ref_heights_control_point.setChecked(True)
        self.radioButton_corr_ref_heights_control_point.setObjectName("radioButton_corr_ref_heights_control_point")
        self.buttonGroup_corrections_ref_heights.addButton(self.radioButton_corr_ref_heights_control_point)
        self.verticalLayout_2.addWidget(self.radioButton_corr_ref_heights_control_point)
        self.verticalLayout_4.addLayout(self.verticalLayout_2)
        self.verticalLayout_7.addWidget(self.groupBox_corrections_ref_heights)
        self.groupBox_atm_pres_correction = QtWidgets.QGroupBox(self.tab_all_instruments)
        self.groupBox_atm_pres_correction.setObjectName("groupBox_atm_pres_correction")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.groupBox_atm_pres_correction)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.checkBox_corrections_atm_pressure = QtWidgets.QCheckBox(self.groupBox_atm_pres_correction)
        self.checkBox_corrections_atm_pressure.setChecked(False)
        self.checkBox_corrections_atm_pressure.setObjectName("checkBox_corrections_atm_pressure")
        self.verticalLayout_5.addWidget(self.checkBox_corrections_atm_pressure)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.formLayout_2 = QtWidgets.QFormLayout()
        self.formLayout_2.setObjectName("formLayout_2")
        self.label_atm_pres_admittance = QtWidgets.QLabel(self.groupBox_atm_pres_correction)
        self.label_atm_pres_admittance.setEnabled(False)
        self.label_atm_pres_admittance.setObjectName("label_atm_pres_admittance")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_atm_pres_admittance)
        self.doubleSpinBox_atm_pres_admittance = QtWidgets.QDoubleSpinBox(self.groupBox_atm_pres_correction)
        self.doubleSpinBox_atm_pres_admittance.setEnabled(False)
        self.doubleSpinBox_atm_pres_admittance.setMinimum(-99.99)
        self.doubleSpinBox_atm_pres_admittance.setSingleStep(0.1)
        self.doubleSpinBox_atm_pres_admittance.setProperty("value", 0.3)
        self.doubleSpinBox_atm_pres_admittance.setObjectName("doubleSpinBox_atm_pres_admittance")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBox_atm_pres_admittance)
        self.horizontalLayout.addLayout(self.formLayout_2)
        self.verticalLayout_5.addLayout(self.horizontalLayout)
        self.verticalLayout_7.addWidget(self.groupBox_atm_pres_correction)
        self.groupBox_scale_correction = QtWidgets.QGroupBox(self.tab_all_instruments)
        self.groupBox_scale_correction.setObjectName("groupBox_scale_correction")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.groupBox_scale_correction)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.checkBox_linear_scale_correction = QtWidgets.QCheckBox(self.groupBox_scale_correction)
        self.checkBox_linear_scale_correction.setObjectName("checkBox_linear_scale_correction")
        self.verticalLayout_6.addWidget(self.checkBox_linear_scale_correction)
        self.verticalLayout_7.addWidget(self.groupBox_scale_correction)
        spacerItem1 = QtWidgets.QSpacerItem(20, 19, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_7.addItem(spacerItem1)
        self.tabWidget.addTab(self.tab_all_instruments, "")
        self.tab_cg6_only = QtWidgets.QWidget()
        self.tab_cg6_only.setObjectName("tab_cg6_only")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.tab_cg6_only)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.groupBox_ocean_loading = QtWidgets.QGroupBox(self.tab_cg6_only)
        self.groupBox_ocean_loading.setObjectName("groupBox_ocean_loading")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.groupBox_ocean_loading)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.checkBox_apply_ocean_loading_lynxlg = QtWidgets.QCheckBox(self.groupBox_ocean_loading)
        self.checkBox_apply_ocean_loading_lynxlg.setChecked(True)
        self.checkBox_apply_ocean_loading_lynxlg.setObjectName("checkBox_apply_ocean_loading_lynxlg")
        self.verticalLayout_9.addWidget(self.checkBox_apply_ocean_loading_lynxlg)
        self.verticalLayout_10.addWidget(self.groupBox_ocean_loading)
        spacerItem2 = QtWidgets.QSpacerItem(20, 465, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_10.addItem(spacerItem2)
        self.tabWidget.addTab(self.tab_cg6_only, "")
        self.verticalLayout_8.addWidget(self.tabWidget)
        self.buttonBox = QtWidgets.QDialogButtonBox(Dialog_corrections)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout_8.addWidget(self.buttonBox)

        self.retranslateUi(Dialog_corrections)
        self.tabWidget.setCurrentIndex(0)
        self.buttonBox.accepted.connect(Dialog_corrections.accept) # type: ignore
        self.buttonBox.rejected.connect(Dialog_corrections.reject) # type: ignore
        self.radioButton_corr_tides_time_series.toggled['bool'].connect(self.comboBox_tides_interpolation_method.setEnabled) # type: ignore
        self.radioButton_corr_tides_time_series.toggled['bool'].connect(self.label_tides_interpolation_method.setEnabled) # type: ignore
        self.checkBox_corrections_atm_pressure.toggled['bool'].connect(self.label_atm_pres_admittance.setEnabled) # type: ignore
        self.checkBox_corrections_atm_pressure.toggled['bool'].connect(self.doubleSpinBox_atm_pres_admittance.setEnabled) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Dialog_corrections)

    def retranslateUi(self, Dialog_corrections):
        _translate = QtCore.QCoreApplication.translate
        Dialog_corrections.setWindowTitle(_translate("Dialog_corrections", "Corrections"))
        self.groupBox_corrections_tides.setToolTip(_translate("Dialog_corrections", "<html><head/><body><p>Options for tidal corretio of observation data.</p></body></html>"))
        self.groupBox_corrections_tides.setTitle(_translate("Dialog_corrections", "Tidal correction"))
        self.radioButton_corr_tides_no_correction.setToolTip(_translate("Dialog_corrections", "No tidal corrections applied."))
        self.radioButton_corr_tides_no_correction.setText(_translate("Dialog_corrections", "No correction"))
        self.radioButton_corr_tides_cg5_model.setToolTip(_translate("Dialog_corrections", "<html><head/><body><p>The tidal correction calculated by the instrument is applied. The Scintrex CG-5 Gravimeter uses, e.g., the Longman (1959) model. </p></body></html>"))
        self.radioButton_corr_tides_cg5_model.setText(_translate("Dialog_corrections", "Instrument correction model"))
        self.radioButton_corr_tides_longman1959.setToolTip(_translate("Dialog_corrections", "Tidal corrections are calculated ans applied in GravTools by evaluationg the model by Longman (1959). "))
        self.radioButton_corr_tides_longman1959.setText(_translate("Dialog_corrections", "Longman (1959)"))
        self.radioButton_corr_tides_time_series.setToolTip(_translate("Dialog_corrections", "<html><head/><body><p>Interpolate corrections from time series data. The python module <span style=\" font-style:italic;\">scipy.interpolation.interp1</span> is used.</p></body></html>"))
        self.radioButton_corr_tides_time_series.setText(_translate("Dialog_corrections", "Interpolate from time series"))
        self.label_tides_interpolation_method.setToolTip(_translate("Dialog_corrections", "Method for the interpolation of correction time series."))
        self.label_tides_interpolation_method.setText(_translate("Dialog_corrections", "Interpolation method"))
        self.comboBox_tides_interpolation_method.setToolTip(_translate("Dialog_corrections", "Method for the interpolation of correction time series."))
        self.groupBox_corrections_ref_heights.setToolTip(_translate("Dialog_corrections", "The original gravity reading refer to the location (height) of the sensor within the instrument. By using the vertical differences \"dhb\" and \"dhf\" provided in the CG-5observation files along with the observation data and the vertical gravity gradients (standard value or provided in the station data), the observations can be reduced to different refernce heights."))
        self.groupBox_corrections_ref_heights.setTitle(_translate("Dialog_corrections", "Reference height"))
        self.radioButton_corr_ref_heights_sensor.setToolTip(_translate("Dialog_corrections", "Sensor heigt (measurement heugt => no correction applied)"))
        self.radioButton_corr_ref_heights_sensor.setText(_translate("Dialog_corrections", "Sensor"))
        self.radioButton_corr_ref_heights_instrument_top.setToolTip(_translate("Dialog_corrections", "Top of the gravimeter."))
        self.radioButton_corr_ref_heights_instrument_top.setText(_translate("Dialog_corrections", "Instrument top"))
        self.radioButton_corr_ref_heights_ground.setToolTip(_translate("Dialog_corrections", "Ground level at the station."))
        self.radioButton_corr_ref_heights_ground.setText(_translate("Dialog_corrections", "Ground"))
        self.radioButton_corr_ref_heights_control_point.setToolTip(_translate("Dialog_corrections", "Height of the control point."))
        self.radioButton_corr_ref_heights_control_point.setText(_translate("Dialog_corrections", "Control point"))
        self.groupBox_atm_pres_correction.setToolTip(_translate("Dialog_corrections", "<html><head/><body><p>Corrections of atmospheric pressure variations.</p></body></html>"))
        self.groupBox_atm_pres_correction.setTitle(_translate("Dialog_corrections", "Atmospheric pressure variations"))
        self.checkBox_corrections_atm_pressure.setToolTip(_translate("Dialog_corrections", "<html><head/><body><p>Use measured pressure values (p<span style=\" vertical-align:sub;\">obs, </span>in hPa) provided in the input observation files to correct atmosperhic pressure variations. Be aware that this correction is only applied, if pressure measurements are provided along with the observaiton data. </p><p>The correction p<span style=\" vertical-align:sub;\">corr</span> in µGal is calculated by multiplying the difference between normal air pressure according to ISO 2533:1975 (p<span style=\" vertical-align:sub;\">n</span>) and p<span style=\" vertical-align:sub;\">obs</span> with the admittance factor k defined below: p<span style=\" vertical-align:sub;\">corr</span> = (p<span style=\" vertical-align:sub;\">obs</span> - p<span style=\" vertical-align:sub;\">n</span>) * k</p></body></html>"))
        self.checkBox_corrections_atm_pressure.setText(_translate("Dialog_corrections", "Atmospheric pressure variations (ISO 2533:1975)"))
        self.label_atm_pres_admittance.setToolTip(_translate("Dialog_corrections", "Admittance factor for the correction of atmospheric pressure variations based on measured and normal pressure."))
        self.label_atm_pres_admittance.setText(_translate("Dialog_corrections", "Admittance factor"))
        self.doubleSpinBox_atm_pres_admittance.setToolTip(_translate("Dialog_corrections", "<html><head/><body><p>Admittance factor for the correction of atmospheric pressure variations based on measured and normal pressure.</p></body></html>"))
        self.groupBox_scale_correction.setToolTip(_translate("Dialog_corrections", "<html><head/><body><p>A priori scale factor correction. Scale factors are applied on the uncorrected gravity observations. </p></body></html>"))
        self.groupBox_scale_correction.setTitle(_translate("Dialog_corrections", "Scale corrections"))
        self.checkBox_linear_scale_correction.setToolTip(_translate("Dialog_corrections", "<html><head/><body><p>Applie linear scale factor, as listed in the gravimeter tab, on the uncorrected gravity observations. Scale factors can be loaded from json-formatted gravimeter files. Scale factors have to be available for all observation epochs, if this correction is applied.</p></body></html>"))
        self.checkBox_linear_scale_correction.setText(_translate("Dialog_corrections", "Apply linear scale factors"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_all_instruments), _translate("Dialog_corrections", "All instruments"))
        self.groupBox_ocean_loading.setTitle(_translate("Dialog_corrections", "Ocean loading"))
        self.checkBox_apply_ocean_loading_lynxlg.setToolTip(_translate("Dialog_corrections", "<html><head/><body><p>Correct the observations for ocean loading effects based on the correction provided by LynxLG (tablet). </p></body></html>"))
        self.checkBox_apply_ocean_loading_lynxlg.setText(_translate("Dialog_corrections", "Apply ocean loading corrections from LynxLG (if available)"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_cg6_only), _translate("Dialog_corrections", "Scintrex CG6 only"))
        self.buttonBox.setToolTip(_translate("Dialog_corrections", "When clicking OK the corrections are applied on all observations in the current campaign."))



"""Context menu handler for the survey table.

Copyright (C) 2023  Andreas Hellerschmied <andreas.hellerschmied@bev.gv.at>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

from PyQt5.QtWidgets import QMenu
from PyQt5 import QtGui


class SurveyTableHandler:
    """Context menu handler for the survey table."""

    def __init__(self, parent=None):
        self.parent = parent

    def right_click(self):
        # bar = self.parent.menuBar()
        top_menu = QMenu(self.parent)

        menu = top_menu.addMenu("Menu")
        config = menu.addMenu("Configuration ...")

        _load = config.addAction("&Load ...")
        _save = config.addAction("&Save ...")

        config.addSeparator()

        config1 = config.addAction("Config1")
        config2 = config.addAction("Config2")
        config3 = config.addAction("Config3")

        action = menu.exec_(QtGui.QCursor.pos())

        if action == _load:
            # do this
            pass
        elif action == _save:
            # do this
            pass
        elif action == config1:
            # do this
            pass
        elif action == config2:
            # do this
            pass
        elif action == config3:
            # do this
            pass


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'gravtools/gui/dialog_estimation_settings.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog_estimation_settings(object):
    def setupUi(self, Dialog_estimation_settings):
        Dialog_estimation_settings.setObjectName("Dialog_estimation_settings")
        Dialog_estimation_settings.resize(589, 599)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(Dialog_estimation_settings)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.tabWidget_estimation_settings = QtWidgets.QTabWidget(Dialog_estimation_settings)
        self.tabWidget_estimation_settings.setEnabled(True)
        self.tabWidget_estimation_settings.setObjectName("tabWidget_estimation_settings")
        self.tab_general_settings = QtWidgets.QWidget()
        self.tab_general_settings.setObjectName("tab_general_settings")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.tab_general_settings)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.groupBox_adjustment_settings = QtWidgets.QGroupBox(self.tab_general_settings)
        self.groupBox_adjustment_settings.setObjectName("groupBox_adjustment_settings")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox_adjustment_settings)
        self.verticalLayout.setObjectName("verticalLayout")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.label = QtWidgets.QLabel(self.groupBox_adjustment_settings)
        self.label.setObjectName("label")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label)
        self.comboBox_adjustment_method = QtWidgets.QComboBox(self.groupBox_adjustment_settings)
        self.comboBox_adjustment_method.setObjectName("comboBox_adjustment_method")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.comboBox_adjustment_method)
        self.label_5 = QtWidgets.QLabel(self.groupBox_adjustment_settings)
        self.label_5.setObjectName("label_5")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_5)
        self.lineEdit_comment = QtWidgets.QLineEdit(self.groupBox_adjustment_settings)
        self.lineEdit_comment.setObjectName("lineEdit_comment")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.lineEdit_comment)
        self.verticalLayout.addLayout(self.formLayout)
        self.checkBox_iterative_s0_scaling = QtWidgets.QCheckBox(self.groupBox_adjustment_settings)
        self.checkBox_iterative_s0_scaling.setChecked(False)
        self.checkBox_iterative_s0_scaling.setObjectName("checkBox_iterative_s0_scaling")
        self.verticalLayout.addWidget(self.checkBox_iterative_s0_scaling)
        self.verticalLayout_8.addWidget(self.groupBox_adjustment_settings)
        self.groupBox_parameters = QtWidgets.QGroupBox(self.tab_general_settings)
        self.groupBox_parameters.setObjectName("groupBox_parameters")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox_parameters)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.formLayout_2 = QtWidgets.QFormLayout()
        self.formLayout_2.setObjectName("formLayout_2")
        self.label_4 = QtWidgets.QLabel(self.groupBox_parameters)
        self.label_4.setObjectName("label_4")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_4)
        self.spinBox_degree_drift_polynomial = QtWidgets.QSpinBox(self.groupBox_parameters)
        self.spinBox_degree_drift_polynomial.setMinimum(1)
        self.spinBox_degree_drift_polynomial.setMaximum(3)
        self.spinBox_degree_drift_polynomial.setProperty("value", 1)
        self.spinBox_degree_drift_polynomial.setObjectName("spinBox_degree_drift_polynomial")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.spinBox_degree_drift_polynomial)
        self.verticalLayout_2.addLayout(self.formLayout_2)
        self.verticalLayout_8.addWidget(self.groupBox_parameters)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_8.addItem(spacerItem)
        self.tabWidget_estimation_settings.addTab(self.tab_general_settings, "")
        self.tab_advanced_settings = QtWidgets.QWidget()
        self.tab_advanced_settings.setObjectName("tab_advanced_settings")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.tab_advanced_settings)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.groupBox_constraints = QtWidgets.QGroupBox(self.tab_advanced_settings)
        self.groupBox_constraints.setObjectName("groupBox_constraints")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.groupBox_constraints)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.formLayout_3 = QtWidgets.QFormLayout()
        self.formLayout_3.setObjectName("formLayout_3")
        self.label_3 = QtWidgets.QLabel(self.groupBox_constraints)
        self.label_3.setObjectName("label_3")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_3)
        self.doubleSpinBox_weight_factor_datum = QtWidgets.QDoubleSpinBox(self.groupBox_constraints)
        self.doubleSpinBox_weight_factor_datum.setDecimals(1)
        self.doubleSpinBox_weight_factor_datum.setMaximum(1000000.0)
        self.doubleSpinBox_weight_factor_datum.setProperty("value", 1.0)
        self.doubleSpinBox_weight_factor_datum.setObjectName("doubleSpinBox_weight_factor_datum")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBox_weight_factor_datum)
        self.doubleSpinBox_sig0 = QtWidgets.QDoubleSpinBox(self.groupBox_constraints)
        self.doubleSpinBox_sig0.setMaximum(1000000.0)
        self.doubleSpinBox_sig0.setProperty("value", 1.0)
        self.doubleSpinBox_sig0.setObjectName("doubleSpinBox_sig0")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBox_sig0)
        self.label_sig0 = QtWidgets.QLabel(self.groupBox_constraints)
        self.label_sig0.setObjectName("label_sig0")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_sig0)
        self.verticalLayout_4.addLayout(self.formLayout_3)
        self.verticalLayout_9.addWidget(self.groupBox_constraints)
        self.groupBox_observations = QtWidgets.QGroupBox(self.tab_advanced_settings)
        self.groupBox_observations.setObjectName("groupBox_observations")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.groupBox_observations)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.formLayout_6 = QtWidgets.QFormLayout()
        self.formLayout_6.setObjectName("formLayout_6")
        self.label_2 = QtWidgets.QLabel(self.groupBox_observations)
        self.label_2.setObjectName("label_2")
        self.formLayout_6.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.doubleSpinBox_add_const_sd = QtWidgets.QDoubleSpinBox(self.groupBox_observations)
        self.doubleSpinBox_add_const_sd.setDecimals(1)
        self.doubleSpinBox_add_const_sd.setMinimum(-100.0)
        self.doubleSpinBox_add_const_sd.setObjectName("doubleSpinBox_add_const_sd")
        self.formLayout_6.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBox_add_const_sd)
        self.label_8 = QtWidgets.QLabel(self.groupBox_observations)
        self.label_8.setObjectName("label_8")
        self.formLayout_6.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_8)
        self.doubleSpinBox_mult_factor_sd = QtWidgets.QDoubleSpinBox(self.groupBox_observations)
        self.doubleSpinBox_mult_factor_sd.setDecimals(1)
        self.doubleSpinBox_mult_factor_sd.setProperty("value", 1.0)
        self.doubleSpinBox_mult_factor_sd.setObjectName("doubleSpinBox_mult_factor_sd")
        self.formLayout_6.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBox_mult_factor_sd)
        self.verticalLayout_7.addLayout(self.formLayout_6)
        self.verticalLayout_9.addWidget(self.groupBox_observations)
        self.groupBox_statistical_tests = QtWidgets.QGroupBox(self.tab_advanced_settings)
        self.groupBox_statistical_tests.setObjectName("groupBox_statistical_tests")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.groupBox_statistical_tests)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.formLayout_5 = QtWidgets.QFormLayout()
        self.formLayout_5.setObjectName("formLayout_5")
        self.label_6 = QtWidgets.QLabel(self.groupBox_statistical_tests)
        self.label_6.setObjectName("label_6")
        self.formLayout_5.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_6)
        self.doubleSpinBox_conf_level_chi = QtWidgets.QDoubleSpinBox(self.groupBox_statistical_tests)
        self.doubleSpinBox_conf_level_chi.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.doubleSpinBox_conf_level_chi.setMaximum(1.0)
        self.doubleSpinBox_conf_level_chi.setSingleStep(0.01)
        self.doubleSpinBox_conf_level_chi.setProperty("value", 0.95)
        self.doubleSpinBox_conf_level_chi.setObjectName("doubleSpinBox_conf_level_chi")
        self.formLayout_5.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBox_conf_level_chi)
        self.label_7 = QtWidgets.QLabel(self.groupBox_statistical_tests)
        self.label_7.setObjectName("label_7")
        self.formLayout_5.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_7)
        self.doubleSpinBox_conf_level_tau = QtWidgets.QDoubleSpinBox(self.groupBox_statistical_tests)
        self.doubleSpinBox_conf_level_tau.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.doubleSpinBox_conf_level_tau.setMaximum(1.0)
        self.doubleSpinBox_conf_level_tau.setSingleStep(0.01)
        self.doubleSpinBox_conf_level_tau.setProperty("value", 0.95)
        self.doubleSpinBox_conf_level_tau.setObjectName("doubleSpinBox_conf_level_tau")
        self.formLayout_5.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBox_conf_level_tau)
        self.verticalLayout_6.addLayout(self.formLayout_5)
        self.verticalLayout_9.addWidget(self.groupBox_statistical_tests)
        self.groupBox_drift_polynomial_advanced = QtWidgets.QGroupBox(self.tab_advanced_settings)
        self.groupBox_drift_polynomial_advanced.setObjectName("groupBox_drift_polynomial_advanced")
        self.verticalLayout_14 = QtWidgets.QVBoxLayout(self.groupBox_drift_polynomial_advanced)
        self.verticalLayout_14.setObjectName("verticalLayout_14")
        self.formLayout_9 = QtWidgets.QFormLayout()
        self.formLayout_9.setObjectName("formLayout_9")
        self.label_10 = QtWidgets.QLabel(self.groupBox_drift_polynomial_advanced)
        self.label_10.setObjectName("label_10")
        self.formLayout_9.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_10)
        self.verticalLayout_13 = QtWidgets.QVBoxLayout()
        self.verticalLayout_13.setObjectName("verticalLayout_13")
        self.radioButton_drift_ref_epoch_first_ob_campaign = QtWidgets.QRadioButton(self.groupBox_drift_polynomial_advanced)
        self.radioButton_drift_ref_epoch_first_ob_campaign.setObjectName("radioButton_drift_ref_epoch_first_ob_campaign")
        self.buttonGroup = QtWidgets.QButtonGroup(Dialog_estimation_settings)
        self.buttonGroup.setObjectName("buttonGroup")
        self.buttonGroup.addButton(self.radioButton_drift_ref_epoch_first_ob_campaign)
        self.verticalLayout_13.addWidget(self.radioButton_drift_ref_epoch_first_ob_campaign)
        self.radioButton_drift_ref_epoch_first_obs_survey = QtWidgets.QRadioButton(self.groupBox_drift_polynomial_advanced)
        self.radioButton_drift_ref_epoch_first_obs_survey.setChecked(True)
        self.radioButton_drift_ref_epoch_first_obs_survey.setObjectName("radioButton_drift_ref_epoch_first_obs_survey")
        self.buttonGroup.addButton(self.radioButton_drift_ref_epoch_first_obs_survey)
        self.verticalLayout_13.addWidget(self.radioButton_drift_ref_epoch_first_obs_survey)
        self.formLayout_9.setLayout(0, QtWidgets.QFormLayout.FieldRole, self.verticalLayout_13)
        self.verticalLayout_14.addLayout(self.formLayout_9)
        self.verticalLayout_9.addWidget(self.groupBox_drift_polynomial_advanced)
        self.groupBox_se_determination = QtWidgets.QGroupBox(self.tab_advanced_settings)
        self.groupBox_se_determination.setObjectName("groupBox_se_determination")
        self.verticalLayout_17 = QtWidgets.QVBoxLayout(self.groupBox_se_determination)
        self.verticalLayout_17.setObjectName("verticalLayout_17")
        self.formLayout_11 = QtWidgets.QFormLayout()
        self.formLayout_11.setObjectName("formLayout_11")
        self.label_11 = QtWidgets.QLabel(self.groupBox_se_determination)
        self.label_11.setObjectName("label_11")
        self.formLayout_11.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_11)
        self.doubleSpinBox_gravity_noise_floor_mugal = QtWidgets.QDoubleSpinBox(self.groupBox_se_determination)
        self.doubleSpinBox_gravity_noise_floor_mugal.setDecimals(1)
        self.doubleSpinBox_gravity_noise_floor_mugal.setProperty("value", 5.0)
        self.doubleSpinBox_gravity_noise_floor_mugal.setObjectName("doubleSpinBox_gravity_noise_floor_mugal")
        self.formLayout_11.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBox_gravity_noise_floor_mugal)
        self.verticalLayout_17.addLayout(self.formLayout_11)
        self.verticalLayout_9.addWidget(self.groupBox_se_determination)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_9.addItem(spacerItem1)
        self.tabWidget_estimation_settings.addTab(self.tab_advanced_settings, "")
        self.tab_iterativ_scaling = QtWidgets.QWidget()
        self.tab_iterativ_scaling.setObjectName("tab_iterativ_scaling")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.tab_iterativ_scaling)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.groupBox_iterative_scaling = QtWidgets.QGroupBox(self.tab_iterativ_scaling)
        self.groupBox_iterative_scaling.setObjectName("groupBox_iterative_scaling")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.groupBox_iterative_scaling)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.formLayout_4 = QtWidgets.QFormLayout()
        self.formLayout_4.setObjectName("formLayout_4")
        self.label_9 = QtWidgets.QLabel(self.groupBox_iterative_scaling)
        self.label_9.setObjectName("label_9")
        self.formLayout_4.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_9)
        self.comboBox_iteration_approach = QtWidgets.QComboBox(self.groupBox_iterative_scaling)
        self.comboBox_iteration_approach.setObjectName("comboBox_iteration_approach")
        self.formLayout_4.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.comboBox_iteration_approach)
        self.label_target_s02 = QtWidgets.QLabel(self.groupBox_iterative_scaling)
        self.label_target_s02.setObjectName("label_target_s02")
        self.formLayout_4.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_target_s02)
        self.doubleSpinBox_target_s02 = QtWidgets.QDoubleSpinBox(self.groupBox_iterative_scaling)
        self.doubleSpinBox_target_s02.setProperty("value", 1.0)
        self.doubleSpinBox_target_s02.setObjectName("doubleSpinBox_target_s02")
        self.formLayout_4.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBox_target_s02)
        self.label_delta_target_s02 = QtWidgets.QLabel(self.groupBox_iterative_scaling)
        self.label_delta_target_s02.setObjectName("label_delta_target_s02")
        self.formLayout_4.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_delta_target_s02)
        self.doubleSpinBox_delta_target_s02 = QtWidgets.QDoubleSpinBox(self.groupBox_iterative_scaling)
        self.doubleSpinBox_delta_target_s02.setMaximum(10.0)
        self.doubleSpinBox_delta_target_s02.setSingleStep(0.1)
        self.doubleSpinBox_delta_target_s02.setProperty("value", 0.1)
        self.doubleSpinBox_delta_target_s02.setObjectName("doubleSpinBox_delta_target_s02")
        self.formLayout_4.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBox_delta_target_s02)
        self.label_max_number_of_iterations = QtWidgets.QLabel(self.groupBox_iterative_scaling)
        self.label_max_number_of_iterations.setObjectName("label_max_number_of_iterations")
        self.formLayout_4.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.label_max_number_of_iterations)
        self.spinBox_max_number_of_iterations = QtWidgets.QSpinBox(self.groupBox_iterative_scaling)
        self.spinBox_max_number_of_iterations.setMinimum(1)
        self.spinBox_max_number_of_iterations.setMaximum(100)
        self.spinBox_max_number_of_iterations.setProperty("value", 10)
        self.spinBox_max_number_of_iterations.setObjectName("spinBox_max_number_of_iterations")
        self.formLayout_4.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.spinBox_max_number_of_iterations)
        self.verticalLayout_5.addLayout(self.formLayout_4)
        self.groupBox_iterative_scaling_multiplicative = QtWidgets.QGroupBox(self.groupBox_iterative_scaling)
        self.groupBox_iterative_scaling_multiplicative.setObjectName("groupBox_iterative_scaling_multiplicative")
        self.verticalLayout_12 = QtWidgets.QVBoxLayout(self.groupBox_iterative_scaling_multiplicative)
        self.verticalLayout_12.setObjectName("verticalLayout_12")
        self.formLayout_8 = QtWidgets.QFormLayout()
        self.formLayout_8.setObjectName("formLayout_8")
        self.label_initial_step_size_percent = QtWidgets.QLabel(self.groupBox_iterative_scaling_multiplicative)
        self.label_initial_step_size_percent.setObjectName("label_initial_step_size_percent")
        self.formLayout_8.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_initial_step_size_percent)
        self.doubleSpinBox_initial_step_size_percent = QtWidgets.QDoubleSpinBox(self.groupBox_iterative_scaling_multiplicative)
        self.doubleSpinBox_initial_step_size_percent.setDecimals(1)
        self.doubleSpinBox_initial_step_size_percent.setMinimum(0.0)
        self.doubleSpinBox_initial_step_size_percent.setProperty("value", 20.0)
        self.doubleSpinBox_initial_step_size_percent.setObjectName("doubleSpinBox_initial_step_size_percent")
        self.formLayout_8.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBox_initial_step_size_percent)
        self.doubleSpinBox_max_multiplicative_factor_to_sd_percent = QtWidgets.QDoubleSpinBox(self.groupBox_iterative_scaling_multiplicative)
        self.doubleSpinBox_max_multiplicative_factor_to_sd_percent.setDecimals(1)
        self.doubleSpinBox_max_multiplicative_factor_to_sd_percent.setMinimum(100.0)
        self.doubleSpinBox_max_multiplicative_factor_to_sd_percent.setMaximum(900.0)
        self.doubleSpinBox_max_multiplicative_factor_to_sd_percent.setProperty("value", 200.0)
        self.doubleSpinBox_max_multiplicative_factor_to_sd_percent.setObjectName("doubleSpinBox_max_multiplicative_factor_to_sd_percent")
        self.formLayout_8.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBox_max_multiplicative_factor_to_sd_percent)
        self.label_max_multiplicative_factor_to_sd_percent = QtWidgets.QLabel(self.groupBox_iterative_scaling_multiplicative)
        self.label_max_multiplicative_factor_to_sd_percent.setObjectName("label_max_multiplicative_factor_to_sd_percent")
        self.formLayout_8.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_max_multiplicative_factor_to_sd_percent)
        self.label_min_multiplicative_factor_to_sd_percent = QtWidgets.QLabel(self.groupBox_iterative_scaling_multiplicative)
        self.label_min_multiplicative_factor_to_sd_percent.setObjectName("label_min_multiplicative_factor_to_sd_percent")
        self.formLayout_8.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_min_multiplicative_factor_to_sd_percent)
        self.doubleSpinBox_min_multiplicative_factor_to_sd_percent = QtWidgets.QDoubleSpinBox(self.groupBox_iterative_scaling_multiplicative)
        self.doubleSpinBox_min_multiplicative_factor_to_sd_percent.setMinimum(1.0)
        self.doubleSpinBox_min_multiplicative_factor_to_sd_percent.setMaximum(100.0)
        self.doubleSpinBox_min_multiplicative_factor_to_sd_percent.setProperty("value", 50.0)
        self.doubleSpinBox_min_multiplicative_factor_to_sd_percent.setObjectName("doubleSpinBox_min_multiplicative_factor_to_sd_percent")
        self.formLayout_8.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBox_min_multiplicative_factor_to_sd_percent)
        self.verticalLayout_12.addLayout(self.formLayout_8)
        self.verticalLayout_5.addWidget(self.groupBox_iterative_scaling_multiplicative)
        self.groupBox_iterative_scaling_additive = QtWidgets.QGroupBox(self.groupBox_iterative_scaling)
        self.groupBox_iterative_scaling_additive.setObjectName("groupBox_iterative_scaling_additive")
        self.verticalLayout_11 = QtWidgets.QVBoxLayout(self.groupBox_iterative_scaling_additive)
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.formLayout_7 = QtWidgets.QFormLayout()
        self.formLayout_7.setObjectName("formLayout_7")
        self.label_initial_setp_size = QtWidgets.QLabel(self.groupBox_iterative_scaling_additive)
        self.label_initial_setp_size.setObjectName("label_initial_setp_size")
        self.formLayout_7.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_initial_setp_size)
        self.doubleSpinBox_initial_step_size = QtWidgets.QDoubleSpinBox(self.groupBox_iterative_scaling_additive)
        self.doubleSpinBox_initial_step_size.setMinimum(0.0)
        self.doubleSpinBox_initial_step_size.setMaximum(100.0)
        self.doubleSpinBox_initial_step_size.setProperty("value", 5.0)
        self.doubleSpinBox_initial_step_size.setObjectName("doubleSpinBox_initial_step_size")
        self.formLayout_7.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBox_initial_step_size)
        self.doubleSpinBox_max_additive_const_to_sd = QtWidgets.QDoubleSpinBox(self.groupBox_iterative_scaling_additive)
        self.doubleSpinBox_max_additive_const_to_sd.setMaximum(900.0)
        self.doubleSpinBox_max_additive_const_to_sd.setProperty("value", 20.0)
        self.doubleSpinBox_max_additive_const_to_sd.setObjectName("doubleSpinBox_max_additive_const_to_sd")
        self.formLayout_7.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBox_max_additive_const_to_sd)
        self.label_max_additive_const_to_sd = QtWidgets.QLabel(self.groupBox_iterative_scaling_additive)
        self.label_max_additive_const_to_sd.setObjectName("label_max_additive_const_to_sd")
        self.formLayout_7.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_max_additive_const_to_sd)
        self.verticalLayout_11.addLayout(self.formLayout_7)
        self.verticalLayout_5.addWidget(self.groupBox_iterative_scaling_additive)
        self.verticalLayout_10.addWidget(self.groupBox_iterative_scaling)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_10.addItem(spacerItem2)
        self.tabWidget_estimation_settings.addTab(self.tab_iterativ_scaling, "")
        self.tab_vg_estimation = QtWidgets.QWidget()
        self.tab_vg_estimation.setObjectName("tab_vg_estimation")
        self.verticalLayout_16 = QtWidgets.QVBoxLayout(self.tab_vg_estimation)
        self.verticalLayout_16.setObjectName("verticalLayout_16")
        self.groupBox_vg_polynomial = QtWidgets.QGroupBox(self.tab_vg_estimation)
        self.groupBox_vg_polynomial.setObjectName("groupBox_vg_polynomial")
        self.verticalLayout_15 = QtWidgets.QVBoxLayout(self.groupBox_vg_polynomial)
        self.verticalLayout_15.setObjectName("verticalLayout_15")
        self.formLayout_10 = QtWidgets.QFormLayout()
        self.formLayout_10.setObjectName("formLayout_10")
        self.label_vg_polynomial_degree = QtWidgets.QLabel(self.groupBox_vg_polynomial)
        self.label_vg_polynomial_degree.setObjectName("label_vg_polynomial_degree")
        self.formLayout_10.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_vg_polynomial_degree)
        self.spinBox_vg_polynomial_degree = QtWidgets.QSpinBox(self.groupBox_vg_polynomial)
        self.spinBox_vg_polynomial_degree.setMinimum(1)
        self.spinBox_vg_polynomial_degree.setMaximum(3)
        self.spinBox_vg_polynomial_degree.setObjectName("spinBox_vg_polynomial_degree")
        self.formLayout_10.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.spinBox_vg_polynomial_degree)
        self.label_vg_polynomial_ref_height_offset_m = QtWidgets.QLabel(self.groupBox_vg_polynomial)
        self.label_vg_polynomial_ref_height_offset_m.setObjectName("label_vg_polynomial_ref_height_offset_m")
        self.formLayout_10.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_vg_polynomial_ref_height_offset_m)
        self.doubleSpinBox_vg_polynomial_ref_height_offset_m = QtWidgets.QDoubleSpinBox(self.groupBox_vg_polynomial)
        self.doubleSpinBox_vg_polynomial_ref_height_offset_m.setDecimals(3)
        self.doubleSpinBox_vg_polynomial_ref_height_offset_m.setMaximum(99.999)
        self.doubleSpinBox_vg_polynomial_ref_height_offset_m.setSingleStep(0.001)
        self.doubleSpinBox_vg_polynomial_ref_height_offset_m.setStepType(QtWidgets.QAbstractSpinBox.DefaultStepType)
        self.doubleSpinBox_vg_polynomial_ref_height_offset_m.setObjectName("doubleSpinBox_vg_polynomial_ref_height_offset_m")
        self.formLayout_10.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.doubleSpinBox_vg_polynomial_ref_height_offset_m)
        self.verticalLayout_15.addLayout(self.formLayout_10)
        self.verticalLayout_16.addWidget(self.groupBox_vg_polynomial)
        spacerItem3 = QtWidgets.QSpacerItem(20, 419, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_16.addItem(spacerItem3)
        self.tabWidget_estimation_settings.addTab(self.tab_vg_estimation, "")
        self.verticalLayout_3.addWidget(self.tabWidget_estimation_settings)

        self.retranslateUi(Dialog_estimation_settings)
        self.tabWidget_estimation_settings.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(Dialog_estimation_settings)

    def retranslateUi(self, Dialog_estimation_settings):
        _translate = QtCore.QCoreApplication.translate
        Dialog_estimation_settings.setWindowTitle(_translate("Dialog_estimation_settings", "Estimation settings"))
        self.groupBox_adjustment_settings.setTitle(_translate("Dialog_estimation_settings", "Adjustment settings"))
        self.label.setToolTip(_translate("Dialog_estimation_settings", "Adjustment method."))
        self.label.setText(_translate("Dialog_estimation_settings", "Method"))
        self.comboBox_adjustment_method.setToolTip(_translate("Dialog_estimation_settings", "Adjustment method."))
        self.label_5.setToolTip(_translate("Dialog_estimation_settings", "Optional comment on the LSM run."))
        self.label_5.setText(_translate("Dialog_estimation_settings", "Comment"))
        self.lineEdit_comment.setToolTip(_translate("Dialog_estimation_settings", "Optional comment on the LSM run."))
        self.checkBox_iterative_s0_scaling.setText(_translate("Dialog_estimation_settings", "Adjust SD of setup observations to scale a posteriori s0"))
        self.groupBox_parameters.setTitle(_translate("Dialog_estimation_settings", "Estimated general parameters"))
        self.label_4.setToolTip(_translate("Dialog_estimation_settings", "Degree of the drift polynomial estimated for each survey."))
        self.label_4.setText(_translate("Dialog_estimation_settings", "Degree of drift Polynomials"))
        self.spinBox_degree_drift_polynomial.setToolTip(_translate("Dialog_estimation_settings", "Degree of the drift polynomial estimated for each survey."))
        self.tabWidget_estimation_settings.setTabText(self.tabWidget_estimation_settings.indexOf(self.tab_general_settings), _translate("Dialog_estimation_settings", "General settings"))
        self.groupBox_constraints.setTitle(_translate("Dialog_estimation_settings", "Datum constraints and weighting"))
        self.label_3.setToolTip(_translate("Dialog_estimation_settings", "Weight factor for datum constraints: The standard deviation of g at datum stations is divided by this factr in order to adjust the weights of the pseudo observations introduced as datum constraints."))
        self.label_3.setText(_translate("Dialog_estimation_settings", "Weight factor for datum"))
        self.doubleSpinBox_weight_factor_datum.setToolTip(_translate("Dialog_estimation_settings", "Weight factor for datum constraints: The standard deviation of g at datum stations is divided by this factor in order to adjust the weights of the pseudo observations introduced as datum constraints."))
        self.doubleSpinBox_sig0.setToolTip(_translate("Dialog_estimation_settings", "A priori standard deviation of the unit weight. The covariance matrix of observations including constraints (Sig_ll) will be divided by sig0²!"))
        self.label_sig0.setToolTip(_translate("Dialog_estimation_settings", "A priori standard deviation of the unit weight. The covariance matrix of observations including constraints (Sig_ll) will be divided by sig0²!"))
        self.label_sig0.setText(_translate("Dialog_estimation_settings", "A priori Sig0 [µGal]"))
        self.groupBox_observations.setTitle(_translate("Dialog_estimation_settings", "Manipulation of a priori SD of setup observations"))
        self.label_2.setToolTip(_translate("Dialog_estimation_settings", "The defined additive constant is added to the standard deviation (SD) of setup observations in order to scale the SD and the resulting weights to realistic values. The multiplicative scaling factor is applied first!"))
        self.label_2.setText(_translate("Dialog_estimation_settings", "Additive const. to SD [µGal]"))
        self.doubleSpinBox_add_const_sd.setToolTip(_translate("Dialog_estimation_settings", "The defined additive constant is added to the standard deviation (SD) of setup observations in order to scale the SD and the resulting weights to realistic values. The multiplicative scaling factor is applied first!"))
        self.label_8.setToolTip(_translate("Dialog_estimation_settings", "Multiplicative factor for scaling the standard deviation (SD) of setup observations."))
        self.label_8.setText(_translate("Dialog_estimation_settings", "Multiplicative factor for SD"))
        self.doubleSpinBox_mult_factor_sd.setToolTip(_translate("Dialog_estimation_settings", "Multiplicative factor for scaling the standard deviation (SD) of setup observations."))
        self.groupBox_statistical_tests.setTitle(_translate("Dialog_estimation_settings", "Statistical tests"))
        self.label_6.setToolTip(_translate("Dialog_estimation_settings", "Confidence level for the goodness-of-fit test (Chi²)."))
        self.label_6.setText(_translate("Dialog_estimation_settings", "Confidence level Chi²-test"))
        self.doubleSpinBox_conf_level_chi.setToolTip(_translate("Dialog_estimation_settings", "Confidence level for the goodness-of-fit test (Chi²)."))
        self.label_7.setToolTip(_translate("Dialog_estimation_settings", "Confidence level for the outlier detection test (tau-test)."))
        self.label_7.setText(_translate("Dialog_estimation_settings", "Confidence level tau-test"))
        self.doubleSpinBox_conf_level_tau.setToolTip(_translate("Dialog_estimation_settings", "Confidence level for the outlier detection test (tau-test)."))
        self.groupBox_drift_polynomial_advanced.setTitle(_translate("Dialog_estimation_settings", "Drift Polynomial"))
        self.label_10.setToolTip(_translate("Dialog_estimation_settings", "The refernce epoch for the estimated drift polynomials (one per survey) is either the epoch of the first observation on the campaign (same reference epoch for all surveys) or of the first observat in each survey of the campaign (different for each survey)."))
        self.label_10.setText(_translate("Dialog_estimation_settings", "Reference epoch t0"))
        self.radioButton_drift_ref_epoch_first_ob_campaign.setToolTip(_translate("Dialog_estimation_settings", "The epoch of the first observation in campaign is the reference epoch for the estimate drift polynomials for ALL surveys in the campaign. "))
        self.radioButton_drift_ref_epoch_first_ob_campaign.setText(_translate("Dialog_estimation_settings", "First observation in campaign"))
        self.radioButton_drift_ref_epoch_first_obs_survey.setToolTip(_translate("Dialog_estimation_settings", "The epoch of the first observation in each survey is used as reference epoch for the estimated drift polynomial (different for each survey in the campaign)."))
        self.radioButton_drift_ref_epoch_first_obs_survey.setText(_translate("Dialog_estimation_settings", "First observation of survey"))
        self.groupBox_se_determination.setToolTip(_translate("Dialog_estimation_settings", "Settings for the determination of uncertainties of the estimates."))
        self.groupBox_se_determination.setTitle(_translate("Dialog_estimation_settings", "Standard error determination"))
        self.label_11.setToolTip(_translate("Dialog_estimation_settings", "<html><head/><body><p>The standard error <span style=\" font-style:italic;\">SE</span> of the estimated gravity values at stations is calculated by <span style=\" font-style:italic;\">SE = sqrt(SD</span><span style=\" font-style:italic; vertical-align:super;\">2 </span><span style=\" font-style:italic;\">+ NF</span><span style=\" font-style:italic; vertical-align:super;\">2</span><span style=\" font-style:italic;\">)</span>, where <span style=\" font-style:italic;\">SD</span> is the estimated standard deviation of the station\'s gravity and <span style=\" font-style:italic;\">NF</span> is the noise floor value specified here.</p></body></html>"))
        self.label_11.setText(_translate("Dialog_estimation_settings", "Gravity noise floor [µGal]"))
        self.doubleSpinBox_gravity_noise_floor_mugal.setToolTip(_translate("Dialog_estimation_settings", "<html><head/><body><p>The standard error <span style=\" font-style:italic;\">SE</span> of the estimated gravity values at stations is calculated by <span style=\" font-style:italic;\">SE = sqrt(SD</span><span style=\" font-style:italic; vertical-align:super;\">2 </span><span style=\" font-style:italic;\">+ NF</span><span style=\" font-style:italic; vertical-align:super;\">2</span><span style=\" font-style:italic;\">)</span>, where <span style=\" font-style:italic;\">SD</span> is the estimated standard deviation of the station\'s gravity and <span style=\" font-style:italic;\">NF</span> is the noise floor value specified here.</p></body></html>"))
        self.tabWidget_estimation_settings.setTabText(self.tabWidget_estimation_settings.indexOf(self.tab_advanced_settings), _translate("Dialog_estimation_settings", "Advanced  settings"))
        self.groupBox_iterative_scaling.setTitle(_translate("Dialog_estimation_settings", "Iterative scaling of SD of setup observations"))
        self.label_9.setToolTip(_translate("Dialog_estimation_settings", "Iteration approach."))
        self.label_9.setText(_translate("Dialog_estimation_settings", "Iteration approach"))
        self.comboBox_iteration_approach.setToolTip(_translate("Dialog_estimation_settings", "Iteration approach."))
        self.label_target_s02.setToolTip(_translate("Dialog_estimation_settings", "Target variance of unit weight for the iteration."))
        self.label_target_s02.setText(_translate("Dialog_estimation_settings", "Target s0² [ µGal²]"))
        self.doubleSpinBox_target_s02.setToolTip(_translate("Dialog_estimation_settings", "Target variance of unit weight for the iteration."))
        self.label_delta_target_s02.setToolTip(_translate("Dialog_estimation_settings", "Permitted deviation of the target variance of unit weight."))
        self.label_delta_target_s02.setText(_translate("Dialog_estimation_settings", "Delta target s0² [ µGal²]"))
        self.doubleSpinBox_delta_target_s02.setToolTip(_translate("Dialog_estimation_settings", "Permitted deviation of the target variance of unit weight."))
        self.label_max_number_of_iterations.setToolTip(_translate("Dialog_estimation_settings", "Maximum number of iterations."))
        self.label_max_number_of_iterations.setText(_translate("Dialog_estimation_settings", "Max. number of iterations"))
        self.spinBox_max_number_of_iterations.setToolTip(_translate("Dialog_estimation_settings", "Maximum number of iterations."))
        self.groupBox_iterative_scaling_multiplicative.setToolTip(_translate("Dialog_estimation_settings", "Settings for the multiplicative iteration approach."))
        self.groupBox_iterative_scaling_multiplicative.setTitle(_translate("Dialog_estimation_settings", "Multiplicative approach"))
        self.label_initial_step_size_percent.setToolTip(_translate("Dialog_estimation_settings", "Intial step size [%] for the iterative scaling of the SD of all setup observations when applying the multiplicative iteration approach."))
        self.label_initial_step_size_percent.setText(_translate("Dialog_estimation_settings", "Initial iteration step size [%]"))
        self.doubleSpinBox_initial_step_size_percent.setToolTip(_translate("Dialog_estimation_settings", "Intial step size [%] for the iterative scaling of the SD of all setup observations when applying the multiplicative iteration approach."))
        self.doubleSpinBox_max_multiplicative_factor_to_sd_percent.setToolTip(_translate("Dialog_estimation_settings", "Maximum multiplicative factor when scaling the SD of all setup observations wehen using the multiplicative iteration approach."))
        self.label_max_multiplicative_factor_to_sd_percent.setToolTip(_translate("Dialog_estimation_settings", "Maximum multiplicative factor when scaling the SD of all setup observations wehen using the multiplicative iteration approach."))
        self.label_max_multiplicative_factor_to_sd_percent.setText(_translate("Dialog_estimation_settings", "Max. multiplicative factor to SD [%]"))
        self.label_min_multiplicative_factor_to_sd_percent.setText(_translate("Dialog_estimation_settings", "Min. multiplicative factor to SD [%]"))
        self.groupBox_iterative_scaling_additive.setToolTip(_translate("Dialog_estimation_settings", "Settings for the additive iteration approach."))
        self.groupBox_iterative_scaling_additive.setTitle(_translate("Dialog_estimation_settings", "Additive approach"))
        self.label_initial_setp_size.setToolTip(_translate("Dialog_estimation_settings", "The initial iteration setp size defines how much the SD of all setup observationa is changed initially. Within the iteration process this step size may become smaller in order to reach the s0² target."))
        self.label_initial_setp_size.setText(_translate("Dialog_estimation_settings", "Initial iteration step size [µGal]"))
        self.doubleSpinBox_initial_step_size.setToolTip(_translate("Dialog_estimation_settings", "The initial iteration setp size defines how much the SD of all setup observationa is changed initially. Within the iteration process this step size may become smaller in order to reach the s0² target."))
        self.doubleSpinBox_max_additive_const_to_sd.setToolTip(_translate("Dialog_estimation_settings", "Definines the maximim permitted additive constant to the standard deviations of the setup observations at the end of the iteration process."))
        self.label_max_additive_const_to_sd.setToolTip(_translate("Dialog_estimation_settings", "Definines the maximim permitted additive constant to the standard deviations of the setup observations at the end of the iteration process."))
        self.label_max_additive_const_to_sd.setText(_translate("Dialog_estimation_settings", "Max. additive const. to SD [µGal]"))
        self.tabWidget_estimation_settings.setTabText(self.tabWidget_estimation_settings.indexOf(self.tab_iterativ_scaling), _translate("Dialog_estimation_settings", "Iterative Scaling"))
        self.groupBox_vg_polynomial.setToolTip(_translate("Dialog_estimation_settings", "Settings for the parameterization of the estimated vertical gravity gradient."))
        self.groupBox_vg_polynomial.setTitle(_translate("Dialog_estimation_settings", "Vertical Gradient Polynomial "))
        self.label_vg_polynomial_degree.setToolTip(_translate("Dialog_estimation_settings", "Degree of the estimated vertical gravity gradient polynomial"))
        self.label_vg_polynomial_degree.setText(_translate("Dialog_estimation_settings", "Degree of the VG polynomial"))
        self.spinBox_vg_polynomial_degree.setToolTip(_translate("Dialog_estimation_settings", "Degree of the estimated vertical gravity gradient polynomial"))
        self.label_vg_polynomial_ref_height_offset_m.setToolTip(_translate("Dialog_estimation_settings", "The reference height for the vertical gravity gradient polynomial ist the height of the control point. With this option it is possible to introduce an offset to the reference height."))
        self.label_vg_polynomial_ref_height_offset_m.setText(_translate("Dialog_estimation_settings", "Reference height offset [m]"))
        self.doubleSpinBox_vg_polynomial_ref_height_offset_m.setToolTip(_translate("Dialog_estimation_settings", "The reference height for the vertical gravity gradient polynomial ist the height of the control point. With this option it is possible to introduce an offset to the reference height."))
        self.tabWidget_estimation_settings.setTabText(self.tabWidget_estimation_settings.indexOf(self.tab_vg_estimation), _translate("Dialog_estimation_settings", "VG estimation"))



"""Misc methods for GravTools models.

Copyright (C) 2021  Andreas Hellerschmied <andreas.hellerschmied@bev.gv.at>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

from typing import Tuple
import numpy as np
import sys
from functools import wraps
from time import time
from collections import Counter


def get_nonunique_items(lst: list) -> list:
    """Returns all non-unique items in the input list.

    Parameters
    ----------
    lst: list
        List to be checked for non-unique items

    Returns
    -------
    list : Non-unique items.
    """
    return [item for item, count in Counter(lst).items() if count > 1]


def unique_ordered_list(seq: list) -> list:
    """Returns a unique list whilst keeping the order.

    Parameters
    ----------
    seq : list
        Input list.

    Returns
    -------
    list : Unique input list with same order of items.
    """
    seen = set()
    seen_add = seen.add
    return [x for x in seq if not (x in seen or seen_add(x))]


def is_singular_matrix(matrix) -> Tuple[bool, float]:
    """Check if the input matrix is singular and return the condition number.

    Parameters
    ----------
    matrix : np.array()
        Matrix (n,n) to be checked for singularity by checking the rank.

    Returns
    -------
    Tuple[bool, float] : `True`, if the matrix is singular and the matrix condition number.

    """
    cond = np.linalg.cond(matrix)
    if cond < 1 / sys.float_info.epsilon:  # not singular
        return False, cond
    else:  # is singular
        return True, cond


def print_matrix(matrix):
    """Print a matrix nicely.
    Parameters
    ----------
    matrix : np.array
        Input matrix.
    """
    for item in matrix:
        temp = (' '.join(f'{s:10.2f}' for s in item))
        print(temp)


def format_seconds_to_hhmmss(seconds):
    """Convert timespan in seconds to hh:mm:ss format.

    Parameters
    ----------
    seconds : float
        Timespan in seconds.

    Returns
    -------
    String: hh:mm:ss

    """
    hours = seconds // (60*60)
    seconds %= (60*60)
    minutes = seconds // 60
    seconds %= 60
    return f'{hours:02d}:{minutes:02d}:{seconds:02d}'


def time_it(func):
    """Decorator to print the execution time of a function."""
    @wraps(func)
    def _time_it(*args, **kwargs):
        start = int(round(time() * 1000))
        try:
            return func(*args, **kwargs)
        finally:
            end_ = int(round(time() * 1000)) - start
            print(f'time_it==> {func.__name__}: execution time: {end_ if end_ > 0 else 0} ms')
    return _time_it


def conditional_decorator(dec, condition):
    """Generic conditional decorator.

    Parameters
    ----------
    dec : decorator
        Decorator that is executed if `condition` is `True`.
    condition: bool
        If `True`, `dec` is used as decorator.
    """
    def decorator(func):
        if not condition:
            # Return the function unchanged, not decorated.
            return func
        return dec(func)
    return decorator


def numpy_array_set_zero(input_array, atol=np.nan):
    """Set all values to 0.0 which are close to zero.

    Parameters
    ----------
    input_array : `np.array`
        Input numpy array.
    atol : float, optional (default=`np.nan`)
        The absolute tolerance parameter of numpy.isclose (see numpy documentation).
    """
    if np.isnan(atol):
        input_array[np.isclose(input_array, 0.0)] = 0.0
    else:
        input_array[np.isclose(input_array, 0.0, atol=atol)] = 0.0
    return input_array




"""Class for TSF-formatted tidal data.

TSF Files are created e.g. by TSoft

Copyright (C) 2023  Andreas Hellerschmied <andreas.hellerschmied@bev.gv.at>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import re
import datetime as dt
import io
from dataclasses import dataclass
import os

import pandas as pd
from typing import Dict, Literal

from gravtools.tides.tide_data import AbstractTideData


@dataclass
class ChannelMetadata:
    """Class for channel metadata in TSF files."""
    location: str
    instrument_name: str
    data_type: str
    unit: str
    channel_number: int

    @property
    def channel_name(self):
        """Returns the channels name as read from the TSF file"""
        return self.location + ':' + self.instrument_name + ':' + self.data_type


class TSF(AbstractTideData):
    """Class for tidal data loaded from TSF files.

    TSF files are the native format for the TSoft which allows to calculate time-series of synthetic tide for a
    specific station defined by geographical coordinates and height.

    TSoft:
    Download TSoft and its manual: http://seismologie.oma.be/en/downloads/tsoft
    Reference: Michel Van Camp, Paul Vauterin: Tsoft: graphical and interactive software for the analysis of time series
    and Earth tides. Comput. Geosci. 31(5): 631-640 (2005)
    DOI: https://doi.org/10.1016/j.cageo.2004.11.015

    Attributes
    ----------
    _filename : str
        Path and name of the TSF file.
    tfs_format : str
        Format version of the TSF file (from block: [TSF-file]). Only version v01.0 is supported!
    timeformat : str
        Format of the time reference (from block: [TIMEFORMAT]). Has to be "DATETIME" or "DATETIMEFRAC"!
    undetval : float
        Determines which number is used to denote undetermined or unknown values in the [DATA] block (from block:
        [UNDETVAL]).
    increment : float
        Determines the time delay (in seconds) between two consecutive data points (from block: [INCREMENT]).
    channel_metadata : list of ChannelMetadata objects
        List of ChannelMetadata objects with one item per channel. Contains metadata such as location, instrument name,
        data type und unit.
    data_df : pandas.DataFrame
        Contains all columns from the [DATA] block. Additionally, the column `epoch_dt` describes the reference time and
        date in as DateTime object.
    countinfo : int, optiona (default=-1)
        Specifies the number of data points in the file (from block: [COUNTINFO]). If this block is missing the in the
        TSF file the default value -1 is set.
    comment : str, optional (default='')
        Optional multiline comment (from block: [COMMENT]).
    """
    def __init__(self, filename: str, tfs_format: str, timeformat: str, undetval: float, increment: float,
                 channel_metadata: list, data_df: pd.DataFrame, countinfo: int = -1, comment: str = ''):
        """Default constructor."""
        self._filename = filename
        self.tfs_format = tfs_format
        self.timeformat = timeformat
        self.undetval = undetval
        self.increment = increment
        self.countinfo = countinfo
        self.comment = comment
        self.channel_metadata = channel_metadata
        self.data_df = data_df


    @classmethod
    def from_tfs_file(cls, filename):
        """ Create class instance from TSF file.

        Parameters
        ----------
        filename : str
            Path and name of TSF file.

        Returns
        -------
        Class instance
        """
        with open(filename, 'r') as f:
            tsf_str = f.read()

            # [TSF-file]: single attribute, single occurrence
            expr = r'\[TSF-file\]\s*(?P<tsf_format>\S+)\s*\n'
            expr_count = 0
            for expr_result in re.finditer(expr, tsf_str):
                expr_result_dict = expr_result.groupdict()
                expr_count += 1
            if expr_count == 1:  # OK
                tfs_format = expr_result_dict['tsf_format']
                if tfs_format != 'v01.0':
                    raise RuntimeError(f'Invalid TSF file format: {tfs_format} (valid: v01.0)')
            else:
                raise RuntimeError(f'TSF-file format tag is missing at the begin of the file, or it occurs more than '
                                   f'once!')

            # [TIMEFORMAT]: single attribute, single occurrence
            expr = r'\[TIMEFORMAT\]\s*(?P<timeformat>\S+)\s*\n'
            expr_count = 0
            for expr_result in re.finditer(expr, tsf_str):
                expr_result_dict = expr_result.groupdict()
                expr_count += 1
            if expr_count == 1:  # OK
                timeformat = expr_result_dict['timeformat']
            elif expr_count == 0:
                raise RuntimeError(f'TSF block [TIMEFORMAT] not found in input TSF file!')
            else:
                raise RuntimeError(f'TSF block [TIMEFORMAT] occurs {expr_count} time! Only once allowed.')
            if timeformat not in ('DATETIME', 'DATETIMEFRAC'):
                raise RuntimeError(f'Invalif [TIMEFORMAT] in TSF File: {timeformat}')

            # [UNDETVAL]: single attribute, single occurrence
            expr = r'\[UNDETVAL\]\s*(?P<undetval>\S+)\s*\n'
            expr_count = 0
            for expr_result in re.finditer(expr, tsf_str):
                expr_result_dict = expr_result.groupdict()
                expr_count += 1
            if expr_count == 1:  # OK
                undetval = expr_result_dict['undetval']
            elif expr_count == 0:
                raise RuntimeError(f'TSF block [UNDETVAL] not found in input TSF file!')
            else:
                raise RuntimeError(f'TSF block [UNDETVAL] occurs {expr_count} time! Only once allowed.')
            undetval = float(undetval)

            # [INCREMENT]: single attribute, single occurrence
            expr = r'\[INCREMENT\]\s*(?P<increment>\S+)\s*\n'
            expr_count = 0
            for expr_result in re.finditer(expr, tsf_str):
                expr_result_dict = expr_result.groupdict()
                expr_count += 1
            if expr_count == 1:  # OK
                increment = expr_result_dict['increment']
            elif expr_count == 0:
                raise RuntimeError(f'TSF block [INCREMENT] not found in input TSF file!')
            else:
                raise RuntimeError(f'TSF block [INCREMENT] occurs {expr_count} time! Only once allowed.')
            increment = float(increment)

            # [COUNTINFO]: optional, single attribute, single occurrence
            expr = r'\[COUNTINFO\]\s*(?P<countinfo>\S+)\s*\n'
            expr_count = 0
            for expr_result in re.finditer(expr, tsf_str):
                expr_result_dict = expr_result.groupdict()
                expr_count += 1
            if expr_count == 1:  # OK
                countinfo = int(expr_result_dict['countinfo'])
            else:
                countinfo = -1  # Error code, if the block wasn't found!

            # [COMMENT]: optional, single attribute, single occurrence
            expr = r'\[COMMENT\]\n(?P<comment>[\s\S]*?(?=\n+\[))'
            expr_count = 0
            for expr_result in re.finditer(expr, tsf_str):
                expr_result_dict = expr_result.groupdict()
                expr_count += 1
            if expr_count == 1:  # OK
                comment = expr_result_dict['comment']
                comment = comment.strip()
            else:
                comment = ''

            # [CHANNELS]: multiple lines, single occurrence
            expr = r'\[CHANNELS\]\n(?P<channels>(?: *.+[\n\r]{0,1})+)\s*'
            expr_count = 0
            for expr_result in re.finditer(expr, tsf_str):
                expr_result_dict = expr_result.groupdict()
                expr_count += 1
            if expr_count == 1:  # OK
                channels = expr_result_dict['channels']
            elif expr_count == 0:
                raise RuntimeError(f'TSF block [CHANNELS] not found in input TSF file!')
            else:
                raise RuntimeError(f'TSF block [CHANNELS] occurs {expr_count} time! Only once allowed.')
            channels = channels.splitlines()
            channels = [s.strip() for s in channels]

            # [UNITS]: multiple lines, single occurrence
            expr = r'\[UNITS\]\n(?P<units>(?: *.+[\n\r]{0,1})+)\s*'
            expr_count = 0
            for expr_result in re.finditer(expr, tsf_str):
                expr_result_dict = expr_result.groupdict()
                expr_count += 1
            if expr_count == 1:  # OK
                units = expr_result_dict['units']
            elif expr_count == 0:
                raise RuntimeError(f'TSF block [UNITS] not found in input TSF file!')
            else:
                raise RuntimeError(f'TSF block [UNITS] occurs {expr_count} time! Only once allowed.')
            units = units.splitlines()
            units = [s.strip() for s in units]

            # Create channel metadata:
            channel_metadata = []
            for count, channel in enumerate(channels):
                channel_metadata.append(ChannelMetadata(location=channel.split(':')[0],
                                instrument_name=channel.split(':')[1],
                                data_type=channel.split(':')[2],
                                unit=units[count],
                                channel_number=count+1))

            # [DATA] multiple blocks possible, multiple lines per block
            data_df_list = []
            if timeformat == 'DATETIME':  # Multiple of seconds
                expr = r'\[DATA\]\n(?P<obs_data>(?:[0-9]{4} [0-9]{2} [0-9]{2}  [0-9]{2} [0-9]{2} [0-9]{2}.+[\n\r]{0,1})+)\s*'
                for expr_result in re.finditer(expr, tsf_str):
                    expr_result_dict = expr_result.groupdict()
                    obs_data_str = expr_result_dict['obs_data']
                    data_df_list.append(pd.read_csv(io.StringIO(obs_data_str), delim_whitespace=True, header=None))
                    columns = ['year', 'month', 'day', 'hour', 'minute', 'second'] + [ch.channel_name for ch in
                                                                                channel_metadata]
                    num_channels_in_data_block = data_df_list[0].shape[1] - 6

            elif timeformat == 'DATETIMEFRAC':
                expr = r'\[DATA\]\n(?P<obs_data>(?:[0-9]{4} [0-9]{2} [0-9]{2}  [0-9]{2} [0-9]{2} [0-9]{2} [0-9]{3}.+[\n\r]{0,1})+)\s*'
                for expr_result in re.finditer(expr, tsf_str):
                    expr_result_dict = expr_result.groupdict()
                    obs_data_str = expr_result_dict['obs_data']
                    data_df_list.append(pd.read_csv(io.StringIO(obs_data_str), delim_whitespace=True, header=None))
                    columns = ['year', 'month', 'day', 'hour', 'minute', 'second','ms'] + [ch.channel_name for ch in
                                                                                           channel_metadata]
                    num_channels_in_data_block = data_df_list[0].shape[1] - 7

            # Concat DATA blocks and add column names:
            if len(data_df_list) == 0:
                raise RuntimeError(f'No [DATA] block found in TSF file!')
            elif len(data_df_list) > 0:
                data_df = pd.concat(data_df_list, ignore_index=True)
                data_df.columns = columns

            # Check, if metadata is available for each channel:
            if num_channels_in_data_block != len(channel_metadata):
                raise RuntimeError(f'Number of data channels in [DATA] block ({num_channels_in_data_block}) not equal '
                                   f'to number of channel metadata '
                                   f'sets in the [CHANNELS] and [UNITS] blocks ({len(channel_metadata)})!')

            # Create datetime column:
            if timeformat == 'DATETIME':
                data_df['epoch_dt'] = pd.to_datetime(data_df[['year', 'month', 'day', 'hour', 'minute', 'second']])
            elif timeformat == 'DATETIMEFRAC':
                data_df['epoch_dt'] = pd.to_datetime(data_df[['year', 'month', 'day', 'hour', 'minute', 'second',
                                                              'ms']])

        return cls(filename=filename, tfs_format=tfs_format, timeformat=timeformat, undetval=undetval,
                   increment=increment, countinfo=countinfo, comment=comment, channel_metadata=channel_metadata,
                   data_df=data_df)

        # super().__init__(filename, filetype='tfs')

    @property
    def get_tide_corr_df(self):
        """Returns the complete tidal gravity time-series as pandas DataFrame"""
        return self.data_df


    @property
    def filename(self) -> str:
        """Returns the name of the data file."""
        return self._filename

    @property
    def filetype(self) -> str:
        """Returns the type of the data file."""
        return 'TSoft TSF file'

    @property
    def number_channels(self) -> str:
        """Returns the number of data channels."""
        return len(self.channel_metadata)

    @property
    def number_records(self) -> str:
        """Returns the number of data records."""
        return len(self.data_df)

    @property
    def starttime(self) -> dt.datetime:
        """Returns the start time of the tidal timeseries."""
        return self.data_df['epoch_dt'].min()

    @property
    def endtime(self) -> dt.datetime:
        """Returns the end time of the tidal timeseries."""
        return self.data_df['epoch_dt'].max()

    def get_channel_df(self, channel: [int, str]):
        """Return the specified channel and the time reference (DateTime) as pandas DataFrame.

        Parameters
        ----------
        channel : str or int or 'last'
            If `channel` is as string the argument is interpreted as channel name and the according channel data is
            returned. If `channel` is an integer, it is interpreted as the number of the channel to be returned. `last`
            indicates that the last channel in the TSF file should be returned.

        Returns
        -------
        pandas.DataFrame containing the data records of the selected channel and the time reference as DateTime.
        """
        if isinstance(channel, str):
            if channel == 'last':
                channel = max([num.channel_number for num in self.channel_metadata])
            elif channel in self.data_df.columns and channel in self.channel_names:
                return self.data_df.loc[:, ['epoch_dt', channel]]
        if isinstance(channel, int):
            for ch in self.channel_metadata:
                if ch.channel_number == channel:
                    return self.data_df.loc[:, ['epoch_dt', ch.channel_name]]
        raise RuntimeError(f'"{channel}" is not a valid TSF data record channel name or number!')

    def get_channel_np(self, channel: [int, str]):
        """Return the time reference and the data of the specified channel as numpy array.

        Parameters
        ----------
        channel : str or int or 'last'
            If `channel` is as string the argument is interpreted as channel name and the according channel data is
            returned. If `channel` is an integer, it is interpreted as the number of the channel to be returned. `last`
            indicates that the last channel in the TSF file should be returned.

        Returns
        -------
        numpy.array of reference epochs (DateTime), numpy.array of channel data.
        """
        if isinstance(channel, str):
            if channel == 'last':
                channel = max([num.channel_number for num in self.channel_metadata])
            elif channel in self.data_df.columns and channel in self.channel_names:
                return self.data_df.loc[:, 'epoch_dt'].to_numpy(), self.data_df.loc[:, channel].to_numpy()
        if isinstance(channel, int):
            for ch in self.channel_metadata:
                if ch.channel_number == channel:
                    return self.data_df.loc[:, 'epoch_dt'].to_numpy(), self.data_df.loc[:, ch.channel_name].to_numpy()
        raise RuntimeError(f'"{channel}" is not a valid TSF data record channel name or number!')


    @property
    def channel_names(self):
        """Returns a list of the names of all data channels."""
        return [ch.channel_name for ch in self.channel_metadata]

    @property
    def channel_names(self):
        """Returns a list of the names of all data channels."""
        return [ch.channel_name for ch in self.channel_metadata]

    @property
    def locations(self):
        """Returns a list of the locations of all stations."""
        return [ch.location for ch in self.channel_metadata]

    @property
    def instruments(self):
        """Returns a list of the instruments of all stations."""
        return [ch.instrument_name for ch in self.channel_metadata]

    @property
    def data_types(self):
        """Returns a list of the data types of all stations."""
        return [ch.data_type for ch in self.channel_metadata]

    @property
    def units(self):
        """Returns a list of the units of all stations."""
        return [ch.unit for ch in self.channel_metadata]

    @property
    def filename_without_path(self):
        """Returns the TSF filename without path."""
        return os.path.basename(self.filename)









"""Dialog for exporting results.

Copyright (C) 2023  Andreas Hellerschmied <andreas.hellerschmied@bev.gv.at>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

from PyQt5.QtWidgets import QDialog
from PyQt5.QtCore import Qt, pyqtSlot

from gravtools.gui.dialog_export_results import Ui_Dialog_export_results

class DialogExportResults(QDialog, Ui_Dialog_export_results):
    """Dialog for data export."""

    def __init__(self, campaign, has_geopandas=False, parent=None):
        super().__init__(parent)

        self._has_geopandas = has_geopandas

        # Get data:
        self._lsm_runs = campaign.lsm_runs
        self.flag_observation_data_available = len(campaign.surveys) > 0
        self.flag_lsm_runs_available = len(campaign.lsm_run_times) > 0
        # Run the .setupUi() method to show the GUI
        self.setupUi(self)
        # Populate the combo box to select an LSM run (enable/disable groupBoxes accordingly):
        self.comboBox_select_lsm_run.clear()
        self.comboBox_select_lsm_run.addItems(['Current observation selection (no LSM run)'] + campaign.lsm_run_times)

        # Set the lineEdit with the export path:
        self.label_export_path_show.setText(campaign.output_directory)

        # Set initial item in comboBox (last entry/LSM run):
        idx = self.comboBox_select_lsm_run.count() - 1  # Index of last lsm run
        self.comboBox_select_lsm_run.setCurrentIndex(idx)

        # Select LSM run, etc.
        if self.flag_lsm_runs_available:
            lsm_run_idx = idx - 1
        else:
            lsm_run_idx = -1

        # Enable/disable GUI items and add lsm run comment:
        self.write_lsm_run_comment_to_gui(lsm_run_idx)
        self.enable_gui_widgets_based_on_lsm_run_selection(lsm_run_idx)

        # connect signals and slots:
        self.comboBox_select_lsm_run.currentIndexChanged.connect(self.on_comboBox_select_lsm_run_current_index_changed)

        # Optional dependency for GIS data export:
        if self._has_geopandas:
            self.groupBox_gis.setEnabled(True)
        else:
            self.groupBox_gis.setEnabled(False)
            self.checkBox_gis_write_shapefile.setChecked(Qt.Unchecked)

    @pyqtSlot(int)
    def on_comboBox_select_lsm_run_current_index_changed(self, index: int):
        """Invoked whenever the index of the selected item in the combobox changed."""
        lsm_run_idx = index - 1
        self.write_lsm_run_comment_to_gui(lsm_run_idx)
        self.enable_gui_widgets_based_on_lsm_run_selection(lsm_run_idx)

    def write_lsm_run_comment_to_gui(self, lsm_run_idx):
        """Writes the lsm run comment of the selected lsm run to the GUI line edit."""
        if lsm_run_idx > -1:
            self.label_export_comment_show.setText(self.get_lsm_run_comment(lsm_run_idx))
        else:
            self.label_export_comment_show.setText('')

    def get_lsm_run_comment(self, lsm_run_idx: int):
        """Returns the lsm run comment of the run with the specified index."""
        try:
            return self._lsm_runs[lsm_run_idx].comment
        except:
            return ''

    def enable_gui_widgets_based_on_lsm_run_selection(self, lsm_run_idx: int):
        """Enable or disable GUI elements based on the lsm run selection."""

        flag_lsm_run_selected = self.flag_lsm_runs_available and (lsm_run_idx > -1)

        if flag_lsm_run_selected:
            self.groupBox_other_files.setEnabled(True)
            self.groupBox_nsb_file.setEnabled(True)
            self.groupBox_observation_list.setEnabled(True)
            if self._has_geopandas:
                self.groupBox_gis.setEnabled(True)
        else:
            self.groupBox_gis.setEnabled(False)
            self.groupBox_other_files.setEnabled(False)
            self.groupBox_nsb_file.setEnabled(False)
            self.groupBox_observation_list.setEnabled(False)
            if self.flag_observation_data_available:
                self.groupBox_observation_list.setEnabled(True)
        if flag_lsm_run_selected or self.flag_observation_data_available:
            self.buttonBox.buttons()[0].setEnabled(True)  # OK button in buttonBox
        else:
            self.buttonBox.buttons()[0].setEnabled(False)  # OK button in buttonBox

        if flag_lsm_run_selected:  # => lsm_run_idx >= 0
            lsm_method = self._lsm_runs[lsm_run_idx].lsm_method
            if lsm_method == 'LSM_diff' or lsm_method == 'LSM_non_diff' or lsm_method == 'MLR_BEV':  # network adjust.
                self.groupBox_nsb_file.setEnabled(True)
                self.checkBox_save_vg_plot_png.setEnabled(False)
            elif lsm_method == 'VG_LSM_nondiff':  # VG estimation
                self.groupBox_nsb_file.setEnabled(False)
                self.checkBox_save_vg_plot_png.setEnabled(True)
            else:
                raise AssertionError(f'Invalid LSM method: {lsm_method}!')


"""User defined exceptions.

Copyright (C) 2021  Andreas Hellerschmied <andreas.hellerschmied@bev.gv.at>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""


class Error(Exception):
    """Base class for exceptions."""
    pass


class InvaliFileContentError(Error):
    """Exception raised for invalid file content.

    Parameters
    -----------
    message : str
        Explanation of the error
    """

    def __init__(self, message):
        self.message = message
        super().__init__(self.message)


class FileTypeError(Error):
    """Exception raised for invalid file types.

    Parameters
    -----------
    message : str
        Explanation of the error
    valid_file_types : list of str
        List of strings declaring the allowed file types.
    """

    def __init__(self, message, valid_file_types=None):
        self.message = message
        if valid_file_types is not None:
            append_str = '\nThe following file types are valid:\n'
            append_str = append_str + '\n'.join(valid_file_types)
            self.message = self.message + append_str
        super().__init__(self.message)



"""Model classes for PyQt's model view architecture.

Copyright (C) 2021  Andreas Hellerschmied <andreas.hellerschmied@bev.gv.at>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

from PyQt5.QtCore import QAbstractTableModel, Qt
from PyQt5.QtWidgets import QMessageBox

NONE_REPRESENTATION_IN_TABLE_VIEW = ''  # Representation of None values in table views in the GUI


class ResultsVGModel(QAbstractTableModel):
    """Model for displaying the drift-related results."""

    _DECIMAL_PLACES_PER_FLOAT_COLUMN = {
        'coefficient': 3,
        'sd_coeff': 3,
    }

    # Column names (keys) and short description (values):
    _PLOT_COLUMNS_DICT = {
        'degree': 'Degree',
        'coefficient': 'Coefficient',
        'sd_coeff': 'SD',
        'coeff_unit': 'Unit',
    }
    _PLOT_COLUMNS = list(_PLOT_COLUMNS_DICT.keys())  # Actual list of columns to be shown

    def __init__(self, lsm_runs):
        """Initialize the VG results table view model.

        Parameters
        ----------
        lsm_runs : list of py.obj:`gravtools.lsm.LSM` objects
        """
        QAbstractTableModel.__init__(self)
        self._lsm_runs = []
        self._data = None
        self.load_lsm_runs(lsm_runs)
        self._lsm_run_index = None
        self._data_column_names = None

    def load_lsm_runs(self, lsm_runs: list):
        """Load adjustment results.

        Notes
        -----
        The data is assigned by reference.
        """
        self._lsm_runs = lsm_runs

    def update_view_model(self, lsm_run_index: int):
        """Update the `_data` DataFrame that hold the actual data that is displayed."""
        flag_error_init = False
        if lsm_run_index == -1:  # No data available => Invalid index => Reset model data
            flag_error_init = True
        else:
            try:
                results_vg_df = self._lsm_runs[lsm_run_index].get_results_vg_df
            except KeyError:
                QMessageBox.critical(self.parent(), 'Error!', f'LSM run with index "{lsm_run_index}" not found!')
            else:
                if results_vg_df is None:  # E.g. no results in campaign yet
                    flag_error_init = True
                else:
                    self._lsm_run_index = lsm_run_index
                    self._data = results_vg_df.loc[:, self._PLOT_COLUMNS].copy(deep=True)
                    self._data_column_names = self._data.columns.to_list()
        if flag_error_init:
            self._data = None
            self._lsm_run_index = None
            self._data_column_names = None

    def rowCount(self, parent=None):
        if self._data is not None:
            return self._data.shape[0]
        else:
            return 0

    def columnCount(self, parent=None):
        if self._data is not None:
            return self._data.shape[1]
        else:
            return 0

    def data(self, index, role=Qt.DisplayRole):
        if index.isValid():
            if role == Qt.DisplayRole or role == Qt.UserRole:
                value = self._data.iloc[index.row(), index.column()]
                column_name = self._data_column_names[index.column()]
                # Custom formatter (string is expected as return type):
                if value is None:  #
                    return NONE_REPRESENTATION_IN_TABLE_VIEW
                elif isinstance(value, float):
                    if value != value:  # True, if value is "NaN"
                        return NONE_REPRESENTATION_IN_TABLE_VIEW
                    else:
                        if column_name in self._DECIMAL_PLACES_PER_FLOAT_COLUMN.keys():
                            num_dec_places = self._DECIMAL_PLACES_PER_FLOAT_COLUMN[column_name]
                            return '{1:.{0}f}'.format(num_dec_places, value)
                        else:
                            return str(value)
                else:  # all other
                    return str(value)

            if role == Qt.TextAlignmentRole:
                value = self._data.iloc[index.row(), index.column()]
                if isinstance(value, int) or isinstance(value, float):
                    # Align right, vertical middle.
                    return Qt.AlignVCenter + Qt.AlignRight
        return None

    def headerData(self, section, orientation, role):
        # section is the index of the column/row.
        if role == Qt.DisplayRole:
            if self._data is not None:
                if orientation == Qt.Horizontal:
                    return self._PLOT_COLUMNS_DICT[str(self._data.columns[section])]
                if orientation == Qt.Vertical:
                    return str(self._data.index[section])

    def get_short_column_description(self, column_name: str) -> str:
        """Returns the short description of the model column."""
        try:
            return self._PLOT_COLUMNS_DICT[column_name]
        except AttributeError:
            return ''
